query getSources($sourceType: SourceTypeFilter) {
  sources(sourceType: $sourceType) {
    nodes {
      fullName
      sourceDbName
      sourceDbVersion
      drugClaimsCount
      geneClaimsCount
      interactionClaimsCount
      license
      licenseLink
    }
  }
}
