"""Provide tools for network-based operations.

For our purposes, drugs and genes are the only nodes on the network, because they're the
only real "entities". Groupings like categories and attributes are defined as properties
of the nodes themselves, but different kinds of processing methods and frameworks might
want to break them out as distinct nodes that relate logically to drugs and genes.
"""
