query getGeneCategories($names: [String!]) {
  genes(names: $names) {
    nodes {
      name
      conceptId
      longName
      geneCategoriesWithSources {
        name
        sourceNames
      }
    }
  }
}
